@@ -33,8 +33,8 @@ class StackSettings(BaseSettings):
         "multiple_open_directory",
     ]
 
-    def __init__(self, json_path):
-        super().__init__(json_path)
+    def __init__(self, json_path, profile_name="default"):
+        super().__init__(json_path, profile_name)
         self.chosen_components_widget = None
         self.keep_chosen_components = False
         self.components_parameters_dict: typing.Dict[int, ROIExtractionProfile] = {}
@@ -61,7 +61,7 @@ class StackSettings(BaseSettings):
         self._additional_layers = result.additional_layers
         self.additional_layers_changed.emit()
         self.last_executed_algorithm = result.parameters.algorithm
-        self.set(f"algorithms.{result.parameters.algorithm}", result.parameters.values)
+        self.set_algorithm(f"algorithms.{result.parameters.algorithm}", result.parameters.values)
         self._set_roi_info(result.roi_info, True, [], parameters_dict)
         if result.points is not None:
             self.points = result.points
@@ -78,8 +78,13 @@ class StackSettings(BaseSettings):
         file_name = self.file_save_name()
         res = []
         for i in components:
-            res.append(path.join(dir_path, f"{file_name}_component{i}.tif"))
-            res.append(path.join(dir_path, f"{file_name}_component{i}_mask.tif"))
+            res.extend(
+                (
+                    path.join(dir_path, f"{file_name}_component{i}.tif"),
+                    path.join(dir_path, f"{file_name}_component{i}_mask.tif"),
+                )
+            )
+
         return res
 
     def chosen_components(self) -> typing.List[int]:
@@ -149,7 +154,6 @@ class StackSettings(BaseSettings):
         ):
             # This line clean segmentation also
             self.image = data.image
-        # self.blockSignals(signals)
         state = self.get_project_info()
         # TODO Remove repetition this and set_segmentation code
 
