@@ -31,7 +31,6 @@ class WaitingDialog(QDialog):
             thread.error_signal.connect(self.error_catch)
 
     def error_catch(self, error):
-        # print(self.thread() == QApplication.instance().thread(), error, isinstance(error, Exception))
         self._error_caught = True
         self.reject()
         if self.exception_hook:
@@ -46,6 +45,9 @@ class WaitingDialog(QDialog):
             self.accept()
 
     def exec(self):
+        self.exec_()
+
+    def exec_(self):
         self.thread_to_wait.start()
         return super().exec_()
 
