@@ -10,14 +10,18 @@ from queue import Empty
 import numpy as np
 import pytest
 
+from PartSegCore import state_store
 from PartSegCore.algorithm_describe_base import ROIExtractionProfile
-from PartSegCore.analysis import ProjectTuple
+from PartSegCore.analysis import ProjectTuple, SegmentationPipeline, SegmentationPipelineElement
 from PartSegCore.analysis.measurement_base import AreaType, MeasurementEntry, PerComponent
 from PartSegCore.analysis.measurement_calculation import ComponentsNumber, MeasurementProfile, Volume
 from PartSegCore.image_operations import RadiusType
 from PartSegCore.mask.io_functions import MaskProjectTuple
 from PartSegCore.mask_create import MaskProperty
+from PartSegCore.project_info import HistoryElement
 from PartSegCore.roi_info import ROIInfo
+from PartSegCore.segmentation.restartable_segmentation_algorithms import BorderRim, LowerThresholdAlgorithm
+from PartSegCore.segmentation.segmentation_algorithm import ThresholdAlgorithm
 from PartSegImage import Image
 
 
@@ -27,6 +31,11 @@ def data_test_dir():
     return os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))), "test_data")
 
 
+@pytest.fixture(autouse=True)
+def mock_settings_path(tmp_path, monkeypatch):
+    monkeypatch.setattr(state_store, "save_folder", str(tmp_path))
+
+
 @pytest.fixture(scope="module")
 def bundle_test_dir():
     """Return path to directory with test data"""
@@ -62,21 +71,30 @@ def sigint_after_time():
 
 
 @pytest.fixture
-def image():
+def image(tmp_path):
     data = np.zeros([20, 20, 20, 2], dtype=np.uint8)
     data[10:-1, 1:-1, 1:-1, 0] = 20
     data[1:10, 1:-1, 1:-1, 1] = 20
     data[1:-1, 1:5, 1:-1, 1] = 20
     data[1:-1, -5:-1, 1:-1, 1] = 20
 
-    return Image(data, (10 ** -3, 10 ** -3, 10 ** -3), axes_order="ZYXC")
+    return Image(data, (10**-3, 10**-3, 10**-3), axes_order="ZYXC", file_path=str(tmp_path / "test.tiff"))
 
 
 @pytest.fixture
-def image2(image):
+def image2(image, tmp_path):
     data = np.zeros([20, 20, 20, 1], dtype=np.uint8)
     data[10:-1, 1:-1, 1:-1, 0] = 20
-    return image.merge(Image(data, (10 ** -3, 10 ** -3, 10 ** -3), axes_order="ZYXC"), "C")
+    img = image.merge(Image(data, (10**-3, 10**-3, 10**-3), axes_order="ZYXC"), "C")
+    img.file_path = str(tmp_path / "test2.tiff")
+    return img
+
+
+@pytest.fixture
+def image2d(tmp_path):
+    data = np.zeros([20, 20], dtype=np.uint8)
+    data[10:-1, 1:-1] = 20
+    return Image(data, (10**-3, 10**-3), axes_order="YX", file_path=str(tmp_path / "test.tiff"))
 
 
 @pytest.fixture
@@ -107,6 +125,15 @@ def algorithm_parameters():
     return deepcopy(algorithm_parameters)
 
 
+@pytest.fixture
+def roi_extraction_profile():
+    return ROIExtractionProfile(
+        name="test",
+        algorithm=LowerThresholdAlgorithm.get_name(),
+        values=LowerThresholdAlgorithm.get_default_values(),
+    )
+
+
 @pytest.fixture
 def mask_segmentation_parameters():
     return ROIExtractionProfile(
@@ -159,7 +186,7 @@ def stack_segmentation2(stack_image: MaskProjectTuple, mask_segmentation_paramet
     for i, (x, y) in enumerate(itertools.product([0, 20], repeat=2), start=1):
         data[3:-3, x + 4 : x + 16, y + 4 : y + 16] = i
     data = ROIInfo(stack_image.image.fit_array_to_image(data))
-    mask_segmentation_parameters.values["threshold"]["values"]["threshold"] = 110
+    mask_segmentation_parameters.values.threshold.values.threshold = 110
     parameters = {i: deepcopy(mask_segmentation_parameters) for i in range(1, 5)}
     return dataclasses.replace(
         stack_image, roi_info=data, roi_extraction_parameters=parameters, selected_components=[1, 3]
@@ -168,27 +195,105 @@ def stack_segmentation2(stack_image: MaskProjectTuple, mask_segmentation_paramet
 
 @pytest.fixture
 def mask_property():
-    return MaskProperty(RadiusType.NO, 0, RadiusType.NO, 0, False, False, False)
+    return MaskProperty.simple_mask()
+
+
+@pytest.fixture
+def mask_property_non_default():
+    return MaskProperty(
+        dilate=RadiusType.R2D,
+        dilate_radius=10,
+        fill_holes=RadiusType.R3D,
+        max_holes_size=10,
+        save_components=True,
+        clip_to_mask=True,
+        reversed_mask=True,
+    )
 
 
 @pytest.fixture
 def measurement_profiles():
     statistics = [
         MeasurementEntry(
-            "Segmentation Volume",
-            Volume.get_starting_leaf().replace_(area=AreaType.ROI, per_component=PerComponent.No),
+            name="Segmentation Volume",
+            calculation_tree=Volume.get_starting_leaf().replace_(area=AreaType.ROI, per_component=PerComponent.No),
         ),
         MeasurementEntry(
-            "ROI Components Number",
-            ComponentsNumber.get_starting_leaf().replace_(area=AreaType.ROI, per_component=PerComponent.No),
+            name="ROI Components Number",
+            calculation_tree=ComponentsNumber.get_starting_leaf().replace_(
+                area=AreaType.ROI, per_component=PerComponent.No
+            ),
         ),
     ]
     statistics2 = [
         MeasurementEntry(
-            "Mask Volume", Volume.get_starting_leaf().replace_(area=AreaType.Mask, per_component=PerComponent.No)
+            name="Mask Volume",
+            calculation_tree=Volume.get_starting_leaf().replace_(area=AreaType.Mask, per_component=PerComponent.No),
         ),
     ]
-    return MeasurementProfile("statistic1", statistics), MeasurementProfile("statistic2", statistics + statistics2)
+    return MeasurementProfile(name="statistic1", chosen_fields=statistics), MeasurementProfile(
+        name="statistic2", chosen_fields=statistics + statistics2
+    )
+
+
+@pytest.fixture
+def border_rim_profile():
+    return ROIExtractionProfile(
+        name="border_profile", algorithm=BorderRim.get_name(), values=BorderRim.get_default_values()
+    )
+
+
+@pytest.fixture
+def lower_threshold_profile():
+    return ROIExtractionProfile(
+        name="lower_profile",
+        algorithm=LowerThresholdAlgorithm.get_name(),
+        values=LowerThresholdAlgorithm.get_default_values(),
+    )
+
+
+@pytest.fixture
+def mask_threshold_profile():
+    return ROIExtractionProfile(
+        name="mask_profile",
+        algorithm=ThresholdAlgorithm.get_name(),
+        values=ThresholdAlgorithm.get_default_values(),
+    )
+
+
+@pytest.fixture
+def sample_pipeline(border_rim_profile, lower_threshold_profile, mask_property):
+    return SegmentationPipeline(
+        name="sample_pipeline",
+        segmentation=border_rim_profile,
+        mask_history=[SegmentationPipelineElement(segmentation=lower_threshold_profile, mask_property=mask_property)],
+    )
+
+
+@pytest.fixture
+def sample_pipeline2(border_rim_profile, lower_threshold_profile, mask_property):
+    return SegmentationPipeline(
+        name="sample_pipeline2",
+        segmentation=lower_threshold_profile,
+        mask_history=[SegmentationPipelineElement(segmentation=border_rim_profile, mask_property=mask_property)],
+    )
+
+
+@pytest.fixture
+def history_element(image, lower_threshold_profile):
+    roi = np.zeros(image.shape, dtype=np.uint8)
+    roi[0, 2:10] = 1
+    roi[0, 10:-2] = 2
+
+    return HistoryElement.create(
+        roi_info=ROIInfo(roi),
+        mask=None,
+        roi_extraction_parameters={
+            "algorithm_name": lower_threshold_profile.name,
+            "values": lower_threshold_profile.values,
+        },
+        mask_property=MaskProperty.simple_mask(),
+    )
 
 
 def pytest_collection_modifyitems(session, config, items):
