@@ -1,3 +1,5 @@
+# pylint: disable=R0201
+
 import numpy as np
 import pytest
 from qtpy.QtCore import QEvent
@@ -27,6 +29,7 @@ class TestMeasurementWidget:
         assert not widget.recalculate_button.isEnabled()
 
     @pytest.mark.enablethread
+    @pytest.mark.enabledialog
     def test_base(self, qtbot, analysis_segmentation, part_settings):
         widget = MeasurementWidget(part_settings)
         qtbot.addWidget(widget)
@@ -44,6 +47,7 @@ class TestMeasurementWidget:
         assert widget.info_field.rowCount() == 2
 
     @pytest.mark.enablethread
+    @pytest.mark.enabledialog
     def test_base2(self, qtbot, analysis_segmentation2, part_settings):
         widget = MeasurementWidget(part_settings)
         qtbot.addWidget(widget)
@@ -63,6 +67,7 @@ class TestMeasurementWidget:
 
 class TestSimpleMeasurementsWidget:
     @pytest.mark.enablethread
+    @pytest.mark.enabledialog
     def test_base(self, stack_settings, stack_segmentation1, qtbot):
         widget = SimpleMeasurements(stack_settings)
         qtbot.addWidget(widget)
@@ -93,6 +98,7 @@ class TestMeasurementsStorage:
         storage["bb"] = [4, 5, 6], "np", (PerComponent.Yes, AreaType.ROI)
         obj = MeasurementsStorage()
         obj.add_measurements(storage)
+        obj.set_expand(False)
         assert obj.get_size(True) == (2, 2)
         obj.set_expand(True)
         assert obj.get_size(True) == (3, 4)
