@@ -4,9 +4,9 @@ import os
 import shutil
 import sys
 import time
+import warnings
 from glob import glob
 
-import numpy as np
 import pandas as pd
 import pytest
 
@@ -39,8 +39,9 @@ from PartSegCore.image_operations import RadiusType
 from PartSegCore.io_utils import SaveBase
 from PartSegCore.mask_create import MaskProperty
 from PartSegCore.segmentation.noise_filtering import DimensionType
-from PartSegCore.universal_const import UNIT_SCALE, Units
-from PartSegImage import Image, ImageWriter, TiffImageReader
+from PartSegCore.segmentation.restartable_segmentation_algorithms import LowerThresholdFlowAlgorithm
+from PartSegCore.universal_const import Units
+from PartSegImage import TiffImageReader
 
 ENGINE = None if pd.__version__ == "0.24.0" else "openpyxl"
 
@@ -50,45 +51,28 @@ class MocksCalculation:
         self.file_path = file_path
 
 
-@pytest.fixture
-def create_test_data(tmpdir):
-    # for future use
-    spacing = tuple(x / UNIT_SCALE[Units.nm.value] for x in (210, 70, 70))
-    res = []
-    for i in range(8):
-        mask_data = np.zeros((10, 20, 20 + i), dtype=np.uint8)
-        mask_data[1:-1, 2:-2, 2:-2] = 1
-        data = np.zeros(mask_data.shape + (2,), dtype=np.uint16)
-        data[1:-1, 2:-2, 2:-2] = 15000
-        data[2:-2, 3:-3, 3:7] = 33000
-        data[2:-2, 3:-3, -7:-3] = 33000
-        image = Image(data, spacing, "", mask=mask_data, axes_order="ZYXC")
-        ImageWriter.save(image, os.path.join(str(tmpdir), f"file_{i}.tif"))
-        res.append(os.path.join(str(tmpdir), f"file_{i}.tif"))
-        ImageWriter.save_mask(image, os.path.join(str(tmpdir), f"file_{i}_mask.tif"))
-    return res
-
-
 # TODO add check of per component measurements
 
 # noinspection DuplicatedCode
 class TestCalculationProcess:
     @staticmethod
     def create_calculation_plan():
-        parameters = {
-            "channel": 1,
-            "minimum_size": 200,
-            "threshold": {
-                "name": "Base/Core",
-                "values": {
-                    "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
-                    "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+        parameters = LowerThresholdFlowAlgorithm.__argument_class__(
+            **{
+                "channel": 1,
+                "minimum_size": 200,
+                "threshold": {
+                    "name": "Base/Core",
+                    "values": {
+                        "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
+                        "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+                    },
                 },
-            },
-            "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
-            "side_connection": False,
-            "sprawl_type": {"name": "Euclidean", "values": {}},
-        }
+                "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
+                "side_connection": False,
+                "flow_type": {"name": "Euclidean", "values": {}},
+            }
+        )
 
         segmentation = ROIExtractionProfile(name="test", algorithm="Lower threshold with watershed", values=parameters)
         mask_suffix = MaskSuffix(name="", suffix="_mask")
@@ -106,8 +90,8 @@ class TestCalculationProcess:
                 ),
             ),
             MeasurementEntry(
-                "Segmentation Components Number",
-                calculation_tree=Leaf("Components number", area=AreaType.ROI, per_component=PerComponent.No),
+                name="Segmentation Components Number",
+                calculation_tree=Leaf(name="Components number", area=AreaType.ROI, per_component=PerComponent.No),
             ),
         ]
         statistic = MeasurementProfile(name="base_measure", chosen_fields=chosen_fields, name_prefix="")
@@ -122,20 +106,22 @@ class TestCalculationProcess:
 
     @staticmethod
     def create_calculation_plan2():
-        parameters = {
-            "channel": 0,
-            "minimum_size": 200,
-            "threshold": {
-                "name": "Base/Core",
-                "values": {
-                    "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
-                    "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+        parameters = LowerThresholdFlowAlgorithm.__argument_class__(
+            **{
+                "channel": 0,
+                "minimum_size": 200,
+                "threshold": {
+                    "name": "Base/Core",
+                    "values": {
+                        "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
+                        "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+                    },
                 },
-            },
-            "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
-            "side_connection": False,
-            "sprawl_type": {"name": "Euclidean", "values": {}},
-        }
+                "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
+                "side_connection": False,
+                "flow_type": {"name": "Euclidean", "values": {}},
+            }
+        )
 
         segmentation = ROIExtractionProfile(name="test", algorithm="Lower threshold with watershed", values=parameters)
         chosen_fields = [
@@ -152,8 +138,8 @@ class TestCalculationProcess:
                 ),
             ),
             MeasurementEntry(
-                "Segmentation Components Number",
-                calculation_tree=Leaf("Components number", area=AreaType.ROI, per_component=PerComponent.No),
+                name="Segmentation Components Number",
+                calculation_tree=Leaf(name="Components number", area=AreaType.ROI, per_component=PerComponent.No),
             ),
         ]
         statistic = MeasurementProfile(name="base_measure", chosen_fields=chosen_fields, name_prefix="")
@@ -193,20 +179,22 @@ class TestCalculationProcess:
 
     @staticmethod
     def create_calculation_plan3():
-        parameters = {
-            "channel": 1,
-            "minimum_size": 200,
-            "threshold": {
-                "name": "Base/Core",
-                "values": {
-                    "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
-                    "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+        parameters = LowerThresholdFlowAlgorithm.__argument_class__(
+            **{
+                "channel": 1,
+                "minimum_size": 200,
+                "threshold": {
+                    "name": "Base/Core",
+                    "values": {
+                        "core_threshold": {"name": "Manual", "values": {"threshold": 30000}},
+                        "base_threshold": {"name": "Manual", "values": {"threshold": 13000}},
+                    },
                 },
-            },
-            "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
-            "side_connection": False,
-            "sprawl_type": {"name": "Euclidean", "values": {}},
-        }
+                "noise_filtering": {"name": "Gauss", "values": {"dimension_type": DimensionType.Layer, "radius": 1.0}},
+                "side_connection": False,
+                "flow_type": {"name": "Euclidean", "values": {}},
+            }
+        )
 
         segmentation = ROIExtractionProfile(name="test", algorithm="Lower threshold with watershed", values=parameters)
         mask_suffix = MaskSuffix(name="", suffix="_mask")
@@ -224,19 +212,30 @@ class TestCalculationProcess:
                 ),
             ),
             MeasurementEntry(
-                "Segmentation Components Number",
-                calculation_tree=Leaf("Components number", area=AreaType.ROI, per_component=PerComponent.No),
+                name="Segmentation Components Number",
+                calculation_tree=Leaf(name="Components number", area=AreaType.ROI, per_component=PerComponent.No),
             ),
             MeasurementEntry(
-                "Segmentation Volume per component",
-                calculation_tree=Leaf("Volume", area=AreaType.ROI, per_component=PerComponent.Yes),
+                name="Segmentation Volume per component",
+                calculation_tree=Leaf(name="Volume", area=AreaType.ROI, per_component=PerComponent.Yes),
             ),
         ]
         statistic = MeasurementProfile(name="base_measure", chosen_fields=chosen_fields, name_prefix="")
         statistic_calculate = MeasurementCalculate(
             channel=0, units=Units.µm, measurement_profile=statistic, name_prefix=""
         )
-        mask_create = MaskCreate("", MaskProperty(RadiusType.NO, 0, RadiusType.NO, 0, True, False, False))
+        mask_create = MaskCreate(
+            name="",
+            mask_property=MaskProperty(
+                dilate=RadiusType.NO,
+                dilate_radius=0,
+                fill_holes=RadiusType.NO,
+                max_holes_size=0,
+                save_components=True,
+                clip_to_mask=False,
+                reversed_mask=False,
+            ),
+        )
         parameters2 = {
             "channel": 1,
             "minimum_size": 200,
@@ -260,12 +259,12 @@ class TestCalculationProcess:
                 ),
             ),
             MeasurementEntry(
-                "Segmentation Components Number",
-                calculation_tree=Leaf("Components number", area=AreaType.ROI, per_component=PerComponent.No),
+                name="Segmentation Components Number",
+                calculation_tree=Leaf(name="Components number", area=AreaType.ROI, per_component=PerComponent.No),
             ),
             MeasurementEntry(
-                "Mask Volume per component",
-                calculation_tree=Leaf("Volume", area=AreaType.Mask, per_component=PerComponent.Yes),
+                name="Mask Volume per component",
+                calculation_tree=Leaf(name="Volume", area=AreaType.Mask, per_component=PerComponent.Yes),
             ),
         ]
         statistic = MeasurementProfile(name="base_measure2", chosen_fields=chosen_fields[:], name_prefix="aa_")
@@ -274,8 +273,8 @@ class TestCalculationProcess:
         )
         chosen_fields.append(
             MeasurementEntry(
-                "Segmentation Volume per component",
-                calculation_tree=Leaf("Volume", area=AreaType.ROI, per_component=PerComponent.Yes),
+                name="Segmentation Volume per component",
+                calculation_tree=Leaf(name="Volume", area=AreaType.ROI, per_component=PerComponent.Yes),
             )
         )
         statistic = MeasurementProfile(name="base_measure3", chosen_fields=chosen_fields[:], name_prefix="bb_")
@@ -352,7 +351,12 @@ class TestCalculationProcess:
         return CalculationPlan(tree=tree, name="test")
 
     @pytest.mark.parametrize(
-        "mask_op", [MaskUse("test1"), MaskSum("", "test1", "test2"), MaskIntersection("", "test1", "test2")]
+        "mask_op",
+        [
+            MaskUse(name="test1"),
+            MaskSum(name="", mask1="test1", mask2="test2"),
+            MaskIntersection(name="", mask1="test1", mask2="test2"),
+        ],
     )
     def test_mask_op(self, mask_op, data_test_dir, tmpdir):
         plan = self.create_mask_operation_plan(mask_op)
@@ -430,13 +434,13 @@ class TestCalculationProcess:
                 time.sleep(0.1)
             else:
                 break
-        else:
+        else:  # pragma: no cover
             manager.kill_jobs()
             pytest.fail("jobs hanged")
 
         manager.writer.finish()
         if sys.platform == "darwin":
-            time.sleep(2)
+            time.sleep(2)  # pragma: no cover
         else:
             time.sleep(0.4)
         assert os.path.exists(os.path.join(tmpdir, "test.xlsx"))
@@ -446,17 +450,19 @@ class TestCalculationProcess:
             assert os.path.basename(df.name.units[i]) == f"stack1_component{i+1}.tif"
 
     @pytest.mark.filterwarnings("ignore:This method will be removed")
-    def test_full_pipeline_error(self, tmp_path_factory, data_test_dir, monkeypatch):
+    def test_full_pipeline_error(self, tmp_path, data_test_dir, monkeypatch):
         plan = self.create_calculation_plan()
-        data_dir = tmp_path_factory.mktemp("data")
+        data_dir = tmp_path / "data"
+        data_dir.mkdir()
         file_pattern_copy = os.path.join(data_test_dir, "stack1_components", "stack1_component*.tif")
         file_paths = sorted(glob(file_pattern_copy))
         for el in file_paths:
             shutil.copy(el, data_dir)
-            shutil.copy(data_dir / "stack1_component1.tif", data_dir / "stack1_component10.tif")
+        shutil.copy(data_dir / "stack1_component1.tif", data_dir / "stack1_component10.tif")
         file_pattern = os.path.join(data_dir, "stack1_component*[0-9].tif")
         file_paths = sorted(glob(file_pattern))
-        result_dir = tmp_path_factory.mktemp("result")
+        result_dir = tmp_path / "result"
+        result_dir.mkdir()
 
         assert os.path.basename(file_paths[0]) == "stack1_component1.tif"
         calc = Calculation(
@@ -478,7 +484,7 @@ class TestCalculationProcess:
             manager.get_results()
         manager.writer.finish()
         if sys.platform == "darwin":
-            time.sleep(2)
+            time.sleep(2)  # pragma: no cover
         else:
             time.sleep(0.4)
         assert os.path.exists(os.path.join(result_dir, "test.xlsx"))
@@ -513,7 +519,7 @@ class TestCalculationProcess:
             time.sleep(0.1)
             manager.get_results()
         if sys.platform == "darwin":
-            time.sleep(2)
+            time.sleep(2)  # pragma: no cover
         else:
             time.sleep(0.4)
         manager.writer.finish()
@@ -551,7 +557,11 @@ class TestCalculationProcess:
     @pytest.mark.parametrize("save_method", save_dict.values())
     def test_do_calculation_save(self, tmpdir, data_test_dir, file_name, root_type, save_method: SaveBase):
         save_desc = Save(
-            "_test", "", save_method.get_name(), save_method.get_short_name(), save_method.get_default_values()
+            suffix="_test",
+            directory="",
+            algorithm=save_method.get_name(),
+            short_name=save_method.get_short_name(),
+            values=save_method.get_default_values(),
         )
         plan = self.create_simple_plan(root_type, save_desc)
         file_path = os.path.join(data_test_dir, file_name)
@@ -631,9 +641,9 @@ class TestCalculationProcess:
             time.sleep(0.1)
             res = manager.get_results()
             if res.errors:
-                print(res.errors, file=sys.stderr)
+                warnings.warn(str(res.errors))  # pragma: no cover
         if sys.platform == "darwin":
-            time.sleep(2)
+            time.sleep(2)  # pragma: no cover
         else:
             time.sleep(0.4)
         manager.writer.finish()
