@@ -3,8 +3,16 @@
 import itertools
 
 import numpy as np
+import pytest
 
-from PartSegCore.segmentation.border_smoothing import IterativeVoteSmoothing, OpeningSmoothing, VoteSmoothing
+from PartSegCore.segmentation.border_smoothing import (
+    IterativeSmoothingParams,
+    IterativeVoteSmoothing,
+    OpeningSmoothing,
+    OpeningSmoothingParams,
+    VoteSmoothing,
+    VoteSmoothingParams,
+)
 from PartSegCore.segmentation.watershed import NeighType
 
 
@@ -12,21 +20,21 @@ class TestVoteSmoothing:
     def test_cube_sides(self):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 3})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=3))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 4})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 5})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=5))
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 6})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=6))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
         assert np.all(res2 == res)
@@ -34,25 +42,25 @@ class TestVoteSmoothing:
     def test_cube_edges(self):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 6})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=6))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 7})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=7))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 9})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=9))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 10})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=10))
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 13})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=13))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 14})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=14))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
         assert np.all(res2 == res)
@@ -60,25 +68,25 @@ class TestVoteSmoothing:
     def test_cube_vertex(self):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 7})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=7))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 8})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=8))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 11})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=11))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 12})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=12))
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 17})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=17))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 18})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=18))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
         assert np.all(res2 == res)
@@ -86,16 +94,16 @@ class TestVoteSmoothing:
     def test_square_sides(self):
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 2})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=2))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 3})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=3))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.sides, "support_level": 4})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
         assert np.all(res == res2)
@@ -103,18 +111,18 @@ class TestVoteSmoothing:
     def test_square_edges(self):
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 3})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=3))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 4})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=4))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 5})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=5))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": 6})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=6))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
         assert np.all(res == res2)
@@ -122,18 +130,18 @@ class TestVoteSmoothing:
     def test_square_vertex(self):
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 1})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=1))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 3})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=3))
         assert np.all(res == data)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 4})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=4))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 5})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=5))
         assert np.all(res2 == res)
-        res = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": 6})
+        res = VoteSmoothing.smooth(data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=6))
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
         assert np.all(res == res2)
@@ -165,110 +173,118 @@ class TestIterativeVoteSmoothing:
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 3, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=3, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 4, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 5, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=5, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 6, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=6, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
         assert np.all(res2 == res)
 
-    def test_cube_sides_iter(self):
+    @pytest.mark.parametrize("max_steps", range(2, 8))
+    def test_cube_sides_iter_4(self, max_steps: int):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
 
-        for i in range(2, 8):
-            res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.sides, "support_level": 4, "max_steps": i}
-            )
-            res2 = np.copy(data)
-            for pos in itertools.product([2, -3], repeat=3):
+        res = IterativeVoteSmoothing.smooth(
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4, max_steps=max_steps)
+        )
+        res2 = np.copy(data)
+        for pos in itertools.product([2, -3], repeat=3):
+            sign = np.sign(pos)
+            for shift in generate_neighbour_sides(max_steps - 1, 3):
+                res2[calc_cord(pos, sign, shift)] = 0
+        assert np.all(res2 == res)
+
+    @pytest.mark.parametrize("max_steps", range(2, 8))
+    def test_cube_sides_iter_5(self, max_steps: int):
+        data = np.zeros((50, 50, 50), dtype=np.uint8)
+        data[2:-2, 2:-2, 2:-2] = 1
+
+        res = IterativeVoteSmoothing.smooth(
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=5, max_steps=max_steps)
+        )
+        res2 = np.copy(data)
+        for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
+            res2[pos] = 0
+        for ind in [0, 1, 2]:
+            for pos in itertools.product([2, -3], repeat=2):
                 sign = np.sign(pos)
-                for shift in generate_neighbour_sides(i - 1, 3):
-                    res2[calc_cord(pos, sign, shift)] = 0
-            assert np.all(res2 == res), f"Fail  on step {i}"
+                for shift in generate_neighbour_sides(max_steps - 1, 2):
+                    pos2 = list(calc_cord(pos, sign, shift))
+                    pos2.insert(ind, slice(2, -2))
+                    res2[tuple(pos2)] = 0
+        assert np.all(res2 == res)
 
-        for i in range(2, 8):
-            res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.sides, "support_level": 5, "max_steps": i}
-            )
-            res2 = np.copy(data)
-            for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
-                res2[pos] = 0
-            for ind in [0, 1, 2]:
-                for pos in itertools.product([2, -3], repeat=2):
-                    sign = np.sign(pos)
-                    for shift in generate_neighbour_sides(i - 1, 2):
-                        pos2 = list(calc_cord(pos, sign, shift))
-                        pos2.insert(ind, slice(2, -2))
-                        res2[tuple(pos2)] = 0
-            assert np.all(res2 == res), f"Fail  on step {i}"
+    @pytest.mark.parametrize("max_steps", range(2, 8))
+    def test_cube_sides_iter_6(self, max_steps: int):
+        data = np.zeros((50, 50, 50), dtype=np.uint8)
+        data[2:-2, 2:-2, 2:-2] = 1
 
-        for i in range(2, 8):
-            res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.sides, "support_level": 6, "max_steps": i}
-            )
-            res2 = np.zeros(data.shape, dtype=data.dtype)
-            shift = 2 + i
-            p = slice(shift, -shift)
-            res2[p, p, p] = 1
-            assert np.all(res2 == res), f"Fail  on step {i}"
+        res = IterativeVoteSmoothing.smooth(
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=6, max_steps=max_steps)
+        )
+        res2 = np.zeros(data.shape, dtype=data.dtype)
+        shift = 2 + max_steps
+        p = slice(shift, -shift)
+        res2[p, p, p] = 1
+        assert np.all(res2 == res)
 
     def test_cube_edges_base(self):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 6, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=6, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 7, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=7, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 9, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=9, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 10, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=10, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 13, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=13, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 14, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=14, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
@@ -284,17 +300,22 @@ class TestIterativeVoteSmoothing:
 
         for i in range(2, 4):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.edges, "support_level": 7, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=7, max_steps=i)
             )
             assert np.all(res2 == res), f"Fail  on step {i}"
         for support_level in [8, 9, 10, 11, 12]:
-            res2 = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.edges, "support_level": support_level})
+            res2 = VoteSmoothing.smooth(
+                data, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=support_level)
+            )
             for i in range(2, 8):
                 res = IterativeVoteSmoothing.smooth(
-                    data, {"neighbourhood_type": NeighType.edges, "support_level": support_level, "max_steps": i}
+                    data,
+                    IterativeSmoothingParams(
+                        neighbourhood_type=NeighType.edges, support_level=support_level, max_steps=i
+                    ),
                 )
                 res2 = VoteSmoothing.smooth(
-                    res2, {"neighbourhood_type": NeighType.edges, "support_level": support_level}
+                    res2, VoteSmoothingParams(neighbourhood_type=NeighType.edges, support_level=support_level)
                 )
                 assert np.all(res2 == res), f"Fail  on step {i} for support level {support_level}"
 
@@ -302,37 +323,37 @@ class TestIterativeVoteSmoothing:
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 7, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=7, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 8, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=8, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 11, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=11, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 12, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=12, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 17, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=17, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 18, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=18, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[3:-3, 3:-3, 3:-3] = 1
@@ -345,17 +366,22 @@ class TestIterativeVoteSmoothing:
 
         for i in range(2, 4):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.vertex, "support_level": 7, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=7, max_steps=i)
             )
             assert np.all(data == res), f"Fail  on step {i}"
         for support_level in range(8, 19):
-            res2 = VoteSmoothing.smooth(data, {"neighbourhood_type": NeighType.vertex, "support_level": support_level})
+            res2 = VoteSmoothing.smooth(
+                data, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=support_level)
+            )
             for i in range(2, 8):
                 res = IterativeVoteSmoothing.smooth(
-                    data, {"neighbourhood_type": NeighType.vertex, "support_level": support_level, "max_steps": i}
+                    data,
+                    IterativeSmoothingParams(
+                        neighbourhood_type=NeighType.vertex, support_level=support_level, max_steps=i
+                    ),
                 )
                 res2 = VoteSmoothing.smooth(
-                    res2, {"neighbourhood_type": NeighType.vertex, "support_level": support_level}
+                    res2, VoteSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=support_level)
                 )
                 assert np.all(res2 == res), f"Fail  on step {i} for support level {support_level}"
 
@@ -363,22 +389,22 @@ class TestIterativeVoteSmoothing:
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 2, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=2, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 3, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=3, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.sides, "support_level": 4, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
@@ -389,7 +415,7 @@ class TestIterativeVoteSmoothing:
         data[:, 2:-2, 2:-2] = 1
         for i in range(2, 4):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.sides, "support_level": 3, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=3, max_steps=i)
             )
             res2 = np.copy(data)
             for pos in itertools.product([2, -3], repeat=2):
@@ -400,7 +426,7 @@ class TestIterativeVoteSmoothing:
 
         for i in range(2, 8):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.sides, "support_level": 4, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.sides, support_level=4, max_steps=i)
             )
             res2 = np.zeros(data.shape, dtype=data.dtype)
             shift = 2 + i
@@ -412,26 +438,26 @@ class TestIterativeVoteSmoothing:
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 3, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=3, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 4, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=4, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 5, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=5, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.edges, "support_level": 6, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=6, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
@@ -446,12 +472,12 @@ class TestIterativeVoteSmoothing:
             res2[(0,) + pos] = 0
         for _ in range(2, 5):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.edges, "support_level": 4, "max_steps": 1}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=4, max_steps=1)
             )
             assert np.all(res2 == res)
         for i in range(2, 8):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.edges, "support_level": 5, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=5, max_steps=i)
             )
             res2 = np.copy(data)
             for pos in itertools.product([2, -3], repeat=2):
@@ -461,7 +487,7 @@ class TestIterativeVoteSmoothing:
             assert np.all(res == res2)
         for i in range(2, 8):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.edges, "support_level": 6, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.edges, support_level=6, max_steps=i)
             )
             res2 = np.zeros(data.shape, dtype=data.dtype)
             shift = 2 + i
@@ -473,26 +499,26 @@ class TestIterativeVoteSmoothing:
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 1, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=1, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 3, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=3, max_steps=1)
         )
         assert np.all(res == data)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 4, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=4, max_steps=1)
         )
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 5, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=5, max_steps=1)
         )
         assert np.all(res2 == res)
         res = IterativeVoteSmoothing.smooth(
-            data, {"neighbourhood_type": NeighType.vertex, "support_level": 6, "max_steps": 1}
+            data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=6, max_steps=1)
         )
         res2 = np.zeros(data.shape, dtype=data.dtype)
         res2[:, 3:-3, 3:-3] = 1
@@ -507,12 +533,12 @@ class TestIterativeVoteSmoothing:
             res2[(0,) + pos] = 0
         for _ in range(2, 5):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.vertex, "support_level": 4, "max_steps": 1}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=4, max_steps=1)
             )
             assert np.all(res2 == res)
         for i in range(2, 8):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.vertex, "support_level": 5, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=5, max_steps=i)
             )
             res2 = np.copy(data)
             for pos in itertools.product([2, -3], repeat=2):
@@ -522,7 +548,7 @@ class TestIterativeVoteSmoothing:
             assert np.all(res == res2)
         for i in range(2, 8):
             res = IterativeVoteSmoothing.smooth(
-                data, {"neighbourhood_type": NeighType.vertex, "support_level": 6, "max_steps": i}
+                data, IterativeSmoothingParams(neighbourhood_type=NeighType.vertex, support_level=6, max_steps=i)
             )
             res2 = np.zeros(data.shape, dtype=data.dtype)
             shift = 2 + i
@@ -535,13 +561,13 @@ class TestOpeningSmooth:
     def test_cube(self):
         data = np.zeros((50, 50, 50), dtype=np.uint8)
         data[2:-2, 2:-2, 2:-2] = 1
-        res = OpeningSmoothing.smooth(data, {"smooth_border_radius": 1})
+        res = OpeningSmoothing.smooth(data, OpeningSmoothingParams(smooth_border_radius=1))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=3):
             res2[pos] = 0
         assert np.all(res2 == res)
 
-        res = OpeningSmoothing.smooth(data, {"smooth_border_radius": 2})
+        res = OpeningSmoothing.smooth(data, OpeningSmoothingParams(smooth_border_radius=2))
         res2 = np.copy(data)
         for pos in itertools.permutations([2, 2, -3, -3, slice(2, -2)], 3):
             res2[pos] = 0
@@ -550,9 +576,9 @@ class TestOpeningSmooth:
     def test_sides(self):
         data = np.zeros((1, 50, 50), dtype=np.uint8)
         data[:, 2:-2, 2:-2] = 1
-        res = OpeningSmoothing.smooth(data, {"smooth_border_radius": 1})
+        res = OpeningSmoothing.smooth(data, OpeningSmoothingParams(smooth_border_radius=1))
         assert np.all(data == res)
-        res = OpeningSmoothing.smooth(data, {"smooth_border_radius": 2})
+        res = OpeningSmoothing.smooth(data, OpeningSmoothingParams(smooth_border_radius=2))
         res2 = np.copy(data)
         for pos in itertools.product([2, -3], repeat=2):
             res2[(0,) + pos] = 0
